##Library Header
# $Id: $
# Copyright  2003-2009 by IXIA
# All Rights Reserved.
#
# Name:
#    ixia_isis_api.tcl
#
# Purpose:
#     A script development library containing ISIS APIs for test automation
#     with the Ixia chassis.
#
# Author:
#    Brad Leabo
#    Karim Lacasse
#
# Usage:
#    package require Ixia
#
# Description:
#    The procedures contained within this library include:
#
#    - emulation_isis_config
#    - emulation_isis_topology_route_config
#    - emulation_isis_control
#    - emulation_isis_info
#
# Requirements:
#     ixiaapiutils.tcl , a library containing TCL utilities
#     parseddashedargs.tcl , a library containing the proceDescr and
#     parsedashedargds.tcl.
#
# Variables:
#
# Keywords:
#
# Category:
#
################################################################################
#                                                                              #
#                                LEGAL  NOTICE:                                #
#                                ==============                                #
# The following code and documentation (hereinafter "the script") is an        #
# example script for demonstration purposes only.                              #
# The script is not a standard commercial product offered by Ixia and have     #
# been developed and is being provided for use only as indicated herein. The   #
# script [and all modifications, enhancements and updates thereto (whether     #
# made by Ixia and/or by the user and/or by a third party)] shall at all times #
# remain the property of Ixia.                                                 #
#                                                                              #
# Ixia does not warrant (i) that the functions contained in the script will    #
# meet the user's requirements or (ii) that the script will be without         #
# omissions or error-free.                                                     #
# THE SCRIPT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, AND IXIA        #
# DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE,          #
# INCLUDING BUT NOT LIMITED TO ANY WARRANTY OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE OR OF NON-INFRINGEMENT.                                 #
# THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SCRIPT  IS WITH THE #
# USER.                                                                        #
# IN NO EVENT SHALL IXIA BE LIABLE FOR ANY DAMAGES RESULTING FROM OR ARISING   #
# OUT OF THE USE OF, OR THE INABILITY TO USE THE SCRIPT OR ANY PART THEREOF,   #
# INCLUDING BUT NOT LIMITED TO ANY LOST PROFITS, LOST BUSINESS, LOST OR        #
# DAMAGED DATA OR SOFTWARE OR ANY INDIRECT, INCIDENTAL, PUNITIVE OR            #
# CONSEQUENTIAL DAMAGES, EVEN IF IXIA HAS BEEN ADVISED OF THE POSSIBILITY OF   #
# SUCH DAMAGES IN ADVANCE.                                                     #
# Ixia will not be required to provide any software maintenance or support     #
# services of any kind (e.g., any error corrections) in connection with the    #
# script or any part thereof. The user acknowledges that although Ixia may     #
# from time to time and in its sole discretion provide maintenance or support  #
# services for the script, any such services are subject to the warranty and   #
# damages limitations set forth herein and will not obligate Ixia to provide   #
# any additional maintenance or support services.                              #
#                                                                              #
################################################################################


proc ::ixia::emulation_isis_config { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
    
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_isis_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable new_ixnetwork_api
    variable isis_handles_array

    ::ixia::utrackerLog $procName $args

    ### an arbitrary default IPV4 address used for te_router_id
    set default_te_router_id    123.0.0.1
    set IPV4        4
    set IPV6        6

    set man_args {
        -mode           CHOICES create modify delete disable enable
        -port_handle    REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
    }

    ### Renamed optional_args to opt_args.  The ::ixia::parse_dashed_args proc
    ### overwrites the optional_args var.
    set opt_args {
        -area_authentication_mode           CHOICES null text md5
        -area_id                            DEFAULT "49 00 01"
        -area_id_step                       DEFAULT "00 00 00"
        -area_password
        -atm_encapsulation                  CHOICES VccMuxIPV4Routed
                                            CHOICES VccMuxIPV6Routed
                                            CHOICES VccMuxBridgedEthernetFCS
                                            CHOICES VccMuxBridgedEthernetNoFCS
                                            CHOICES LLCRoutedCLIP
                                            CHOICES LLCBridgedEthernetFCS
                                            CHOICES LLCBridgedEthernetNoFCS
        -attach_bit                         CHOICES 0 1
        -bfd_registration                   CHOICES 0 1
                                            DEFAULT 0
        -count                              DEFAULT 1
        -dce_capability_router_id           IPV4
                                            DEFAULT 0.0.0.0
        -dce_bcast_root_priority            RANGE 0-65535 DEFAULT 65535
        -dce_num_mcast_dst_trees            RANGE 0-65535 DEFAULT 1
        -dce_device_id                      RANGE 0-65535 DEFAULT 1
        -dce_device_pri                     RANGE 0-255   DEFAULT 1
        -dce_ftag_enable                    CHOICES 0 1   DEFAULT 0
        -dce_ftag                           RANGE 0-65535 DEFAULT 1
        -discard_lsp                        CHOICES 0 1
        -domain_authentication_mode         CHOICES null text md5
        -domain_password
        -gateway_ip_addr                    IP
                                            DEFAULT 0.0.0.0
        -gateway_ip_addr_step               IP
                                            DEFAULT 0.0.1.0
        -gateway_ipv6_addr                  IPV6
                                            DEFAULT 0::0
        -gateway_ipv6_addr_step             IPV6
                                            DEFAULT 0:0:0:1::0
        -graceful_restart                   CHOICES 0 1
        -graceful_restart_mode              CHOICES normal restarting starting
                                            CHOICES helper
        -graceful_restart_restart_time
        -graceful_restart_version           CHOICES draft3 draft4
        -handle
        -hello_interval                     RANGE   1-65535
        -hello_password                     CHOICES 0 1
        -interface_handle
        -intf_ip_addr                       IP
                                            DEFAULT 178.0.0.1
        -intf_ip_prefix_length              RANGE   1-32
                                            DEFAULT 24
        -intf_ip_addr_step                  IP
                                            DEFAULT 0.0.1.0
        -intf_ipv6_addr                     IPV6
                                            DEFAULT 4000::1
        -intf_ipv6_prefix_length            RANGE   1-128
                                            DEFAULT 64
        -intf_ipv6_addr_step                IPV6
                                            DEFAULT 0:0:0:1::0
        -intf_metric                        RANGE   0-16777215
        -intf_type                          CHOICES broadcast ptop
        -ip_version                         CHOICES 4 6 4_6
                                            DEFAULT 4
        -l1_router_priority                 RANGE   0-255
        -l2_router_priority                 RANGE   0-255
        -loopback_bfd_registration          CHOICES 0 1
                                            DEFAULT 0
        -loopback_ip_addr                   IPV4
        -loopback_ip_addr_step              IPV4
                                            DEFAULT 0.0.0.1
        -loopback_ip_prefix_length          RANGE 0-32
                                            DEFAULT 24
        -loopback_ip_addr_count             NUMERIC
                                            DEFAULT 1
        -loopback_metric                    RANGE   0-16777215
        -loopback_type                      CHOICES broadcast ptop
        -loopback_routing_level             CHOICES L1 L2 L1L2
        -loopback_l1_router_priority        RANGE   0-255
        -loopback_l2_router_priority        RANGE   0-255
        -loopback_te_metric                 RANGE   0-2147483647
        -loopback_te_admin_group            RANGE   0-2147483647
        -loopback_te_max_bw                 NUMERIC
        -loopback_te_max_resv_bw            NUMERIC
        -loopback_te_unresv_bw_priority0    NUMERIC
        -loopback_te_unresv_bw_priority1    NUMERIC
        -loopback_te_unresv_bw_priority2    NUMERIC
        -loopback_te_unresv_bw_priority3    NUMERIC
        -loopback_te_unresv_bw_priority4    NUMERIC
        -loopback_te_unresv_bw_priority5    NUMERIC
        -loopback_te_unresv_bw_priority6    NUMERIC
        -loopback_te_unresv_bw_priority7    NUMERIC
        -loopback_hello_password            CHOICES 0 1
        -lsp_life_time                      RANGE   1-65535
        -lsp_refresh_interval               RANGE   1-65535
        -mac_address_init                   MAC
        -mac_address_step                   MAC 
                                            DEFAULT 0000.0000.0001
        -no_write                           FLAG
        -max_packet_size                    RANGE   576-32832
        -partition_repair                   CHOICES 0 1
        -overloaded                         CHOICES 0 1
        -override_existence_check           CHOICES 0 1
                                            DEFAULT 0
        -override_tracking                  CHOICES 0 1
                                            DEFAULT 0
        -reset                              FLAG
        -routing_level                      CHOICES L1 L2 L1L2
        -system_id
        -system_id_step                     DEFAULT 1
        -te_enable                          CHOICES 0 1
        -te_router_id                       IP
        -te_router_id_step                  IP
                                            DEFAULT 0.0.0.1
        -te_metric                          RANGE   0-2147483647
        -te_admin_group                     RANGE   0-2147483647
        -te_max_bw                          NUMERIC
        -te_max_resv_bw                     NUMERIC
        -te_unresv_bw_priority0             NUMERIC
        -te_unresv_bw_priority1             NUMERIC
        -te_unresv_bw_priority2             NUMERIC
        -te_unresv_bw_priority3             NUMERIC
        -te_unresv_bw_priority4             NUMERIC
        -te_unresv_bw_priority5             NUMERIC
        -te_unresv_bw_priority6             NUMERIC
        -te_unresv_bw_priority7             NUMERIC
        -type                               CHOICES dce_isis_draft_ward_l2_isis_04 isis_l3_routing
                                            DEFAULT isis_l3_routing
        -vlan                               CHOICES 0 1
        -vlan_id                            RANGE   0-4095
        -vlan_id_mode                       CHOICES fixed increment
                                            DEFAULT increment
        -vlan_id_step                       RANGE   0-4096
                                            DEFAULT 1
        -vlan_user_priority                 RANGE   0-7
                                            DEFAULT 0
        -vpi                                RANGE   0-255
                                            DEFAULT 1
        -vci                                RANGE   0-65535
                                            DEFAULT 10
        -vpi_step                           RANGE   0-255
                                            DEFAULT 1
        -vci_step                           RANGE   0-65535
                                            DEFAULT 1
        -wide_metrics                       CHOICES 0 1
                                            DEFAULT 0
        -multi_topology                     CHOICES 0 1
                                            DEFAULT 0
    }

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_isis_config $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_isis_topology_route_config { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
    
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_isis_topology_route_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
        
    variable new_ixnetwork_api
    variable isis_handles_array
    keylset returnList status $::SUCCESS
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set man_args {
        -mode   CHOICES create modify delete
    }

    set opt_args {
        -handle
        -elem_handle
        -type                        CHOICES router grid stub external 
                                     CHOICES dce_mcast_mac_range dce_mcast_ipv4_group_range dce_mcast_ipv6_group_range
                                     CHOICES dce_node_mac_group dce_node_ipv4_group dce_node_ipv6_group
                                     CHOICES dce_network_range dce_outside_link
        -ip_version                  CHOICES 4 6 4_6
                                     DEFAULT 4_6
        -router_system_id
        -router_id                   IP
        -router_area_id
        -router_te                   CHOICES 0 1
        -router_connect
        -link_narrow_metric          RANGE   0-63
                                     DEFAULT 0
        -link_wide_metric            RANGE   0-16777215
                                     DEFAULT 0
        -link_ip_addr                IP
                                     DEFAULT 0.0.0.0
        -link_ip_prefix_length       RANGE   1-32
                                     DEFAULT 24
        -link_ipv6_addr              IP
                                     DEFAULT 3000::0
        -link_ipv6_prefix_length     RANGE   1-128
                                     DEFAULT 64
        -link_multi_topology         CHOICES 1
        -link_enable                 CHOICES 0 1
        -link_te                     CHOICES 0 1
                                     DEFAULT 0
        -link_te_metric              RANGE   0-16777215
                                     DEFAULT 0
        -link_te_max_bw              NUMERIC
        -link_te_max_resv_bw         NUMERIC
        -link_te_unresv_bw_priority0 NUMERIC
        -link_te_unresv_bw_priority1 NUMERIC
        -link_te_unresv_bw_priority2 NUMERIC
        -link_te_unresv_bw_priority3 NUMERIC
        -link_te_unresv_bw_priority4 NUMERIC
        -link_te_unresv_bw_priority5 NUMERIC
        -link_te_unresv_bw_priority6 NUMERIC
        -link_te_unresv_bw_priority7 NUMERIC
        -link_te_admin_group         RANGE 1-2147483647
        -grid_row                    NUMERIC
        -grid_col                    NUMERIC
        -grid_user_wide_metric       CHOICES 0 1
                                     DEFAULT 1
        -grid_stub_per_router        NUMERIC
        -grid_router_count           NUMERIC
        -grid_router_id              IP
        -grid_router_id_step         IP
        -grid_router_ip_version      CHOICES 4 6
                                     DEFAULT 4
        -grid_router_route_step      NUMERIC
        -grid_link_type              CHOICES broadcast ptop
        -grid_ip_start               IP
                                     DEFAULT 0.0.0.0
        -grid_ip_pfx_len             VCMD ::ixia::validate_list_range_1_32
                                     DEFAULT 24
        -grid_ip_step                IP
                                     DEFAULT 0.0.1.0
        -grid_ipv6_start             IP
                                     DEFAULT 3000::0
        -grid_ipv6_pfx_len           VCMD ::ixia::validate_list_range_1_128
                                     DEFAULT 64
        -grid_ipv6_step              IP
                                     DEFAULT 0:0:0:1::0
        -grid_start_te_ip            IP
        -grid_te_ip_step             IP
                                     DEFAULT 0.0.0.1
        -grid_start_system_id        REGEXP  ^[0-9,a-f,A-F]+$
        -grid_system_id_step         REGEXP  ^[0-9,a-f,A-F]+$
                                     DEFAULT 0x000000000001
        -grid_connect                DEFAULT 1 1
        -grid_outside_link           CHOICES 0 1
                                     DEFAULT 0
        -grid_ol_connection_row      NUMERIC
        -grid_ol_connection_col      NUMERIC
        -grid_ol_linked_rid          REGEXP  ^[0-9,a-f,A-F]+$
        -grid_ol_ip_and_prefix         
        -grid_ol_admin_group         RANGE 0-2147483647
        -grid_ol_metric              RANGE 0-16777215
        -grid_ol_max_bw              NUMERIC
        -grid_ol_max_resv_bw         NUMERIC
        -grid_ol_unresv_bw_priority0 DECIMAL        
        -grid_ol_unresv_bw_priority1 DECIMAL
        -grid_ol_unresv_bw_priority2 DECIMAL
        -grid_ol_unresv_bw_priority3 DECIMAL
        -grid_ol_unresv_bw_priority4 DECIMAL
        -grid_ol_unresv_bw_priority5 DECIMAL
        -grid_ol_unresv_bw_priority6 DECIMAL
        -grid_ol_unresv_bw_priority7 DECIMAL
        -grid_te                     CHOICES 0 1
                                     DEFAULT 0
        -grid_router_metric          NUMERIC
        -grid_router_ip_pfx_len      RANGE   1-128
                                     DEFAULT 24
        -grid_router_up_down_bit     CHOICES 0 1
        -grid_router_origin          CHOICES stub external
        -grid_te_admin               RANGE   0-2147483647
        -grid_te_max_bw              NUMERIC
        -grid_te_max_resv_bw         NUMERIC
        -grid_te_metric              NUMERIC
        -grid_te_override_admin               RANGE   0-2147483647
        -grid_te_override_enable              CHOICES 0 1
        -grid_te_override_max_bw              DECIMAL
        -grid_te_override_max_resv_bw         DECIMAL
        -grid_te_override_metric              NUMERIC
        -grid_te_override_unresv_bw_priority0 DECIMAL
        -grid_te_override_unresv_bw_priority1 DECIMAL
        -grid_te_override_unresv_bw_priority2 DECIMAL
        -grid_te_override_unresv_bw_priority3 DECIMAL
        -grid_te_override_unresv_bw_priority4 DECIMAL
        -grid_te_override_unresv_bw_priority5 DECIMAL
        -grid_te_override_unresv_bw_priority6 DECIMAL
        -grid_te_override_unresv_bw_priority7 DECIMAL
        -grid_te_path_count               NUMERIC
                                          DEFAULT 0
        -grid_te_path_start_row           NUMERIC
        -grid_te_path_start_col           NUMERIC
        -grid_te_path_end_row             NUMERIC
        -grid_te_path_end_col             NUMERIC
        -grid_te_path_row_step            NUMERIC
        -grid_te_path_col_step            NUMERIC
        -grid_te_path_bidir               NUMERIC
        -grid_te_path_admin               RANGE   0-2147483647
        -grid_te_path_metric              NUMERIC
        -grid_te_path_max_bw              DECIMAL
        -grid_te_path_max_resv_bw         DECIMAL
        -grid_te_path_unresv_bw_priority0 DECIMAL
        -grid_te_path_unresv_bw_priority1 DECIMAL
        -grid_te_path_unresv_bw_priority2 DECIMAL
        -grid_te_path_unresv_bw_priority3 DECIMAL
        -grid_te_path_unresv_bw_priority4 DECIMAL
        -grid_te_path_unresv_bw_priority5 DECIMAL
        -grid_te_path_unresv_bw_priority6 DECIMAL
        -grid_te_path_unresv_bw_priority7 DECIMAL 
        -grid_te_unresv_bw_priority0 DECIMAL
        -grid_te_unresv_bw_priority1 DECIMAL
        -grid_te_unresv_bw_priority2 DECIMAL
        -grid_te_unresv_bw_priority3 DECIMAL
        -grid_te_unresv_bw_priority4 DECIMAL
        -grid_te_unresv_bw_priority5 DECIMAL
        -grid_te_unresv_bw_priority6 DECIMAL
        -grid_te_unresv_bw_priority7 DECIMAL
        -grid_interface_metric       NUMERIC
        -stub_ip_start               IP
                                     DEFAULT 0.0.0.0
        -stub_ip_pfx_len             RANGE   1-32
                                     DEFAULT 24
        -stub_ip_step                IP
                                     DEFAULT 0.0.1.0
        -stub_ipv6_start             IP
                                     DEFAULT 3000::0
        -stub_ipv6_pfx_len           RANGE   1-128
                                     DEFAULT 64
        -stub_ipv6_step              IP
                                     DEFAULT 0:0:0:1::0
        -stub_count                  NUMERIC
        -stub_route_count            NUMERIC
        -stub_metric                 NUMERIC
        -stub_up_down_bit            CHOICES 0 1
        -stub_connect                DEFAULT 1 1
        -external_ip_start           IP
                                     DEFAULT 0.0.0.0
        -external_ip_pfx_len         RANGE   1-32
                                     DEFAULT 24
        -external_ip_step            IP
                                     DEFAULT 0.0.1.0
        -external_ipv6_start         IP
                                     DEFAULT 3000::0
        -external_ipv6_pfx_len       RANGE   1-128
                                     DEFAULT 64
        -external_ipv6_step          IP
                                     DEFAULT 0:0:0:1::0
        -external_count              NUMERIC
        -external_route_count        NUMERIC
        -external_metric             NUMERIC
        -external_up_down_bit        CHOICES 0 1
        -external_connect            NUMERIC
                                     DEFAULT 1
        -dce_bcast_root_pri                    RANGE 0-65535 DEFAULT 65535
        -dce_bcast_root_pri_step               RANGE 0-65535 DEFAULT 0
        -dce_connection_column                 RANGE 1-100000 DEFAULT 1
        -dce_connection_row                    RANGE 1-100000 DEFAULT 1
        -dce_device_id                         RANGE 0-65535 DEFAULT 1
        -dce_device_id_step                    RANGE 0-65535 DEFAULT 1
        -dce_device_pri                        RANGE 0-255 DEFAULT 1
        -dce_ftag                              RANGE 0-65535 DEFAULT 1
        -dce_ftag_enable                       CHOICES 0 1 DEFAULT 0
        -dce_include_groups                    CHOICES 0 1 DEFAULT 0
        -dce_intra_grp_ucast_step
        -dce_inter_grp_ucast_step
        -dce_linked_router_id
        -dce_local_entry_point_column          RANGE 1-100000 DEFAULT 1
        -dce_local_entry_point_row             RANGE 1-100000 DEFAULT 1
        -dce_local_link_metric                 RANGE 0-63 DEFAULT 1
        -dce_local_num_columns                 RANGE 1-100000 DEFAULT 1
        -dce_local_num_rows                    RANGE 1-100000 DEFAULT 1
        -dce_mcast_addr_count                  RANGE 1-4294967295 DEFAULT 1
        -dce_mcast_addr_node_step
        -dce_mcast_addr_step
        -dce_mcast_start_addr
        -dce_num_mcast_destination_trees       RANGE 0-65535 DEFAULT 1
        -dce_src_grp_mapping                   CHOICES fully_meshed one_to_one manual_mapping
                                               DEFAULT fully_meshed
        -dce_system_id
        -dce_system_id_step
        -dce_ucast_addr_node_step
        -dce_ucast_sources_per_mcast_addr      RANGE 0-4294967295 DEFAULT 1
        -dce_ucast_src_addr
        -dce_vlan_id                           RANGE 0-4095 DEFAULT 1
        -no_write                              FLAG
    }

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_isis_topology_route_config $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_isis_control { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
    
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_isis_control $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable new_ixnetwork_api
    variable isis_handles_array
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set man_args {
        -mode CHOICES start stop restart
    }

    set opt_args {
        -port_handle REGEXP ^[0-9]+/[0-9]+/[0-9]+$
        -handle
    }

	if {[isUNIX] && [info exists ::ixTclSvrHandle]} {
        set retValueClicks [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock clicks}"]
        set retValueSeconds [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock seconds}"]
    } else {
        set retValueClicks [clock clicks]
        set retValueSeconds [clock seconds]
    }
    keylset returnList clicks [format "%u" $retValueClicks]
    keylset returnList seconds [format "%u" $retValueSeconds]

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_isis_control $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        keylset returnList clicks [format "%u" $retValueClicks]
        keylset returnList seconds [format "%u" $retValueSeconds]
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}

    return $returnList
}


proc ::ixia::emulation_isis_info { args } {
    variable executeOnTclServer
    
    set procName [lindex [info level [info level]] 0]
    
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_isis_info $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable new_ixnetwork_api
    variable isis_handles_array
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set man_args {
        -mode        CHOICES stats clear_stats learned_info
    }
    set opt_args {
        -port_handle
        -handle
    }

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_isis_info $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}
